/************************************************************************
 * Lesson22Dbo performs the database input and output for our project.
 * Each instance of this class represents a single row in table
 * PUBLIC.LESSON22.
 * 
 * The class contains static methods for row retrieval and instance
 * methods for inserts and updates.
 * It also has a static method for batch delete, for illustration
 * purposes.
 */
package org.hardknockjava.lesson22.example01;

import java.io.InputStream;
import java.sql.Connection;
import java.sql.Date;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;

public class Lesson22Dbo {
    // Instance variables
    private Integer lineNo;
    private Date entryDate;
    private Timestamp entryTime;
    private String textLine;

    // The following static constants contain the names of table
    // columns and the table name itself, and SQL statements.
    static final String COL_LINE_NO = "LINE_NO";
    static final String COL_ENTRY_DATE = "ENTRY_DATE";
    static final String COL_ENTRY_TIME = "ENTRY_TIME";
    static final String COL_TEXT_LINE = "TEXT_LINE";
    static final String TABLE_NAME = "PUBLIC.LESSON22";

    private static final String STATIC_SELECT_STMT_1 = //
            "SELECT " + COL_LINE_NO //
                    + "\n ," + COL_ENTRY_DATE //
                    + "\n ," + COL_ENTRY_TIME //
                    + "\n ," + COL_TEXT_LINE //
                    + "\n FROM PUBLIC.LESSON22 " //
                    + "\n WHERE " + COL_LINE_NO + " >= ";

    private static final String STATIC_SELECT_STMT_2 = //
            "\n ORDER BY " + COL_LINE_NO;

    private static final String DYNAMIC_SELECT_STMT_BASE = //
            "SELECT " + COL_LINE_NO //
                    + "\n ," + COL_ENTRY_DATE //
                    + "\n ," + COL_ENTRY_TIME //
                    + "\n ," + COL_TEXT_LINE //
                    + "\n FROM " + TABLE_NAME;

    private static final String DYNAMIC_SELECT_ALL_STMT = //
            DYNAMIC_SELECT_STMT_BASE //
                    + "\n WHERE " + COL_LINE_NO + " >= ?" //
                    + "\n ORDER BY " + COL_LINE_NO;

    private static final String DYNAMIC_SELECT_ONE_STMT = //
            DYNAMIC_SELECT_STMT_BASE //
                    + "\n WHERE " + COL_LINE_NO + " = ?" //
                    + "\n ORDER BY " + COL_LINE_NO;

    private static final String INSERT_STMT = //
            "INSERT INTO " + TABLE_NAME //
                    + "\n  (" + COL_LINE_NO //
                    + "\n, " + COL_ENTRY_DATE //
                    + "\n, " + COL_ENTRY_TIME //
                    + "\n, " + COL_TEXT_LINE + ")"//
                    + "\n VALUES (?, ?, ?, ?)";

    private static final String UPDATE_STMT = //
            "UPDATE " + TABLE_NAME //
                    + "\n  SET " + COL_ENTRY_DATE + " = ?" //
                    + "\n    , " + COL_ENTRY_TIME + " = ?" //
                    + "\n    , " + COL_TEXT_LINE + " = ?" //
                    + "\n WHERE " + COL_LINE_NO + " = ?";

    private static final String DELETE_STMT = //
            "DELETE FROM " + TABLE_NAME + " WHERE " + COL_LINE_NO + " >= ?";

    // These static variables are for the Connection and for
    // PreparedStatements. They are not instance specific, and
    // so are defined as static.
    private static Connection conn = null;
    private static PreparedStatement selectAllStmt = null;
    private static PreparedStatement selectOneStmt = null;
    private static PreparedStatement insertStmt = null;
    private static PreparedStatement updateStmt = null;
    private static PreparedStatement deleteStmt = null;

    // Constructor
    public Lesson22Dbo(Integer lineNo, Date entryDate, Timestamp entryTime, String textLine) {
        this.lineNo = lineNo;
        this.entryDate = entryDate;
        this.entryTime = entryTime;
        this.textLine = textLine;
    }

    // Establish the connection and create PreparedStatements.
    public static void connect() throws Exception {
        // Get the URL, user ID, and password from the properties file.
        InputStream is = Lesson22Dbo.class.getClassLoader().getResourceAsStream("lesson22.properties");
        Properties props = new Properties();
        props.load(is);
        String url = props.getProperty("url");
        String user = props.getProperty("user");
        String password = props.getProperty("password");
        conn = DriverManager.getConnection(url, user, password);
        conn.setAutoCommit(false);

        selectAllStmt = conn.prepareStatement(DYNAMIC_SELECT_ALL_STMT);
        selectOneStmt = conn.prepareStatement(DYNAMIC_SELECT_ONE_STMT);
        insertStmt = conn.prepareStatement(INSERT_STMT);
        updateStmt = conn.prepareStatement(UPDATE_STMT);
        deleteStmt = conn.prepareStatement(DELETE_STMT);
    }

    public Integer getLineNo() {
        return lineNo;
    }

    public Date getEntryDate() {
        return entryDate;
    }

    public void setEntryDate(Date entryDate) {
        this.entryDate = entryDate;
    }

    public Timestamp getEntryTime() {
        return entryTime;
    }

    public void setEntryTime(Timestamp entryTime) {
        this.entryTime = entryTime;
    }

    public String getTextLine() {
        return textLine;
    }

    public void setTextLine(String textLine) {
        this.textLine = textLine;
    }

    // Use the static SELECT statement to return all rows.
    public static List<Lesson22Dbo> selectStatic() throws SQLException {
        return selectStatic(Integer.MIN_VALUE);
    }

    // Use the static SELECT statement to return multiple rows starting
    // with the LINE_NO provided.
    public static List<Lesson22Dbo> selectStatic(int firstLine) throws SQLException {
        List<Lesson22Dbo> returnList = new ArrayList<Lesson22Dbo>();

        Statement stmt = conn.createStatement();
        ResultSet rs = null;
        String query = STATIC_SELECT_STMT_1 + firstLine + STATIC_SELECT_STMT_2;
        try {
            rs = stmt.executeQuery(query);
            Lesson22Dbo dbo = null;
            while ((dbo = populateQuery(rs)) != null) {
                returnList.add(dbo);
            }
        } finally {
            if (rs != null) {
                rs.close();
            }
            stmt.close();
        }

        return returnList;
    }

    // Use the prepared SELECT statement to return all rows.
    public static List<Lesson22Dbo> selectAllDynamic() throws SQLException {
        return selectAllDynamic(Integer.MIN_VALUE);
    }

    // Use the prepared SELECT statement to return multiple rows starting
    // with the LINE_NO provided.
    public static List<Lesson22Dbo> selectAllDynamic(int firstLine) throws SQLException {
        List<Lesson22Dbo> returnList = new ArrayList<Lesson22Dbo>();

        selectAllStmt.setInt(1, firstLine);
        displayPreparedStatement(selectAllStmt);
        ResultSet rs = null;

        try {
            rs = selectAllStmt.executeQuery();
            Lesson22Dbo dbo = null;
            while ((dbo = populateQuery(rs)) != null) {
                returnList.add(dbo);
            }
        } finally {
            if (rs != null) {
                rs.close();
            }
        }

        return returnList;
    }

    // Use the prepared SELECT statement to return a single row.
    public static Lesson22Dbo selectOneDynamic(int lineNo) throws SQLException {
        Lesson22Dbo dbo = null;

        selectOneStmt.setInt(1, lineNo);
        displayPreparedStatement(selectOneStmt);
        ResultSet rs = null;

        try {
            rs = selectOneStmt.executeQuery();
            dbo = populateQuery(rs);
        } finally {
            if (rs != null) {
                rs.close();
            }
        }

        return dbo;
    }

    // Create instances from the ResultSet and add them to returnLlist.
    private static Lesson22Dbo populateQuery(ResultSet rs) throws SQLException {
        if (rs.next()) {
            Integer lineNo = rs.getInt(1);
            Date entryDate = rs.getDate(2);
            Timestamp entryTime = rs.getTimestamp(3);
            String textLine = rs.getString(4);
            if (rs.wasNull()) {
                textLine = "(no text here)";
            }
            Lesson22Dbo dbo = new Lesson22Dbo(lineNo, entryDate, entryTime, textLine);
            return dbo;
        } else {
            return null;
        }
    }

    // Dump the list of instances to the console.
    public static void dump(List<Lesson22Dbo> dboList) throws SQLException {
        System.out.println();

        for (Lesson22Dbo dbo : dboList) {
            Integer lineNo = dbo.getLineNo();
            Date entryDate = dbo.getEntryDate();
            Timestamp entryTime = dbo.getEntryTime();
            String textLine = dbo.getTextLine();

            System.out.println(lineNo + " " + entryDate + " " + entryTime + " " + textLine);
        }
    }

    public static void commit() throws SQLException {
        conn.commit();
    }

    public static void rollback() throws SQLException {
        conn.rollback();
    }

    // Insert this instance to the database.
    public void insert() throws SQLException {
        insertStmt.setInt(1, lineNo);
        insertStmt.setDate(2, entryDate);
        insertStmt.setTimestamp(3, entryTime);
        insertStmt.setString(4, textLine);
        displayPreparedStatement(insertStmt);
        insertStmt.execute();
    }

    // Update this instance in the database.
    public int update() throws SQLException {
        updateStmt.setDate(1, entryDate);
        updateStmt.setTimestamp(2, entryTime);
        updateStmt.setString(3, textLine);
        updateStmt.setInt(4, lineNo);
        displayPreparedStatement(updateStmt);
        return updateStmt.executeUpdate();
    }

    // Delete the rows with the provided LINE_NO values as a batch.
    public static int[] batchDelete(List<Integer> lineNoList) throws SQLException {
        for (Integer lineNo : lineNoList) {
            deleteStmt.setInt(1, lineNo);
            deleteStmt.addBatch();
            displayPreparedStatement(deleteStmt);
        }
        return deleteStmt.executeBatch();
    }

    // Display a PreparedStatement on the console.
    public static void displayPreparedStatement(PreparedStatement stmt) throws SQLException {
        System.out.println();
        System.out.println("Prepared statement=" + stmt.toString());
    }

    // Close the PreparedStatements and the Connection.
    public static void close() throws SQLException {
        selectAllStmt.close();
        selectOneStmt.close();
        insertStmt.close();
        updateStmt.close();
        deleteStmt.close();
        deleteStmt = null;
        conn.close();
    }
}