package org.hardknockjava.lesson22.example01;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.Reader;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

public class Lesson22Deleter {
    public static void main(String[] args) throws Exception {
        Lesson22Deleter deleter = new Lesson22Deleter();

        // Open the database connection.
        Lesson22Dbo.connect();
        
        try {
            deleter.process();
        } catch (SQLException e) {
            e.printStackTrace();
            System.out.println("  SQLState=" + e.getSQLState());
            System.out.println("  error code=" + e.getErrorCode());
            System.out.println("  message=" + e.getMessage());
        }

        Lesson22Dbo.close();
    }

    private void process() throws Exception {
        // Set up a reader to read file poem1.txt.
        Reader in = new FileReader("data/poem1.txt");
        BufferedReader rdr = new BufferedReader(in);
        String inputRecord = null;
        
        // Each record of poem1.txt starts with a LINE_NO
        // value. We will delete each row with one of these
        // values.
        
        // Save the LINE_NO values of rows we'll delete from the table.
        List<Integer> lineNoList = new ArrayList<Integer>();
        
        try {
            while ((inputRecord = rdr.readLine()) != null) {
                String[] fields = inputRecord.split(",");
                if (fields.length > 0) {
                    Integer deleteLine = Integer.parseInt(fields[0]);
                    lineNoList.add(deleteLine);
                }
            }

            // Delete all the selected rows in a batch call.
            int[] deleteCounts = Lesson22Dbo.batchDelete(lineNoList);
            int deleteTotal = 0;
            
            // Display the number of rows deleted by each DELETE
            // statement in the batch.
            for (int deleteCount : deleteCounts) {
                System.out.println(deleteCount + " lines were deleted");
                deleteTotal += deleteCount;
            }
            
            // Commit our changes.
            Lesson22Dbo.commit();
            
            System.out.println();
            System.out.println(deleteTotal + " total lines were deleted.");

            // Read all rows remaining in the table and display them on the console.
            List<Lesson22Dbo> dumpList = Lesson22Dbo.selectAllDynamic();
            System.out.println();
            System.out.println("Displaying all rows");
            Lesson22Dbo.dump(dumpList);
        } catch (Exception e) {
            Lesson22Dbo.rollback();
            throw e;
        } finally {
            rdr.close();
        }
    }

}
