package org.hardknockjava.lesson22.example01;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.Reader;
import java.sql.SQLException;
import java.util.List;

public class Lesson22Inserter {
    public static void main(String[] args) throws Exception {
        Lesson22Inserter inserter = new Lesson22Inserter();

        // Open the database connection.
        Lesson22Dbo.connect();

        try {
            inserter.process();
        } catch (SQLException e) {
            SQLException ex = (SQLException) e;
            while (ex != null) {
                ex.printStackTrace();
                System.out.println("  SQLState=" + ex.getSQLState());
                System.out.println("  error code=" + ex.getErrorCode());
                System.out.println("  message=" + ex.getMessage());
                ex = ex.getNextException();
            }
        }

        // Close the database connection.
        Lesson22Dbo.close();
    }

    private void process() throws Exception {
        // Set up a reader to read file poem1.txt.
        Reader in = new FileReader("data/poem1.txt");
        BufferedReader rdr = new BufferedReader(in);
        String inputRecord = null;

        // Keep track of the lowest line number read from the file.
        int firstNewLine = Integer.MAX_VALUE;

        // Each record of the file has a LINE_NO value and
        // a text line, separated by a comma.

        try {
            while ((inputRecord = rdr.readLine()) != null) {
                String[] fields = inputRecord.split(",");
                if (fields.length > 1) {
                    int nextLine = Integer.parseInt(fields[0]);
                    if (nextLine < firstNewLine) {
                        firstNewLine = nextLine;
                    }

                    String line = fields[1];

                    // Create a new DBO using the line number and text
                    // read from the file, and the current date and timestamp.
                    java.util.Date today = new java.util.Date();
                    java.sql.Date newDate = new java.sql.Date(today.getTime());
                    java.sql.Timestamp newTimestamp = new java.sql.Timestamp(today.getTime());
                    Lesson22Dbo dbo = new Lesson22Dbo(nextLine, newDate, newTimestamp, line);

                    // Insert the object we've created.
                    dbo.insert();
                }
            }

            // Commit our changes.
            Lesson22Dbo.commit();

            // Read all the new lines and display them on the console.
            List<Lesson22Dbo> dumpList = Lesson22Dbo.selectAllDynamic(firstNewLine);
            System.out.println();
            System.out.println("Displaying the inserted rows");
            Lesson22Dbo.dump(dumpList);
        } catch (Exception e) {
            Lesson22Dbo.rollback();
            throw e;
        } finally {
            rdr.close();
        }
    }

}
