package org.hardknockjava.lesson22.example01;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.Reader;
import java.sql.SQLException;
import java.util.List;

public class Lesson22Updater {
    public static void main(String[] args) throws Exception {
        Lesson22Updater updater = new Lesson22Updater();

        // Open the database connection.
        Lesson22Dbo.connect();

        try {
            updater.process();
        } catch (SQLException e) {
            e.printStackTrace();
            System.out.println("  SQLState=" + e.getSQLState());
            System.out.println("  error code=" + e.getErrorCode());
            System.out.println("  message=" + e.getMessage());
        }

        // Close the database connection.
        Lesson22Dbo.close();
    }

    private void process() throws Exception {
        // Set up a reader to read file poem2.txt.
        Reader in = new FileReader("data/poem2.txt");
        BufferedReader rdr = new BufferedReader(in);
        String inputRecord = null;

        // Keep track of the lowest line number read from the file.
        int firstUpdateLine = Integer.MAX_VALUE;

        try {
            while ((inputRecord = rdr.readLine()) != null) {
                String[] fields = inputRecord.split(",");
                if (fields.length > 1) {
                    int updateLine = Integer.parseInt(fields[0]);
                    if (updateLine < firstUpdateLine) {
                        firstUpdateLine = updateLine;
                    }

                    String line = fields[1];

                    // Read the row with the selected line number. If it
                    // exists, replace its textLine and update it.
                    Lesson22Dbo dbo = Lesson22Dbo.selectOneDynamic(updateLine);
                    if (dbo != null) {
                        dbo.setTextLine(line);
                        dbo.update();
                    }
                }
            }

            // Commit our changes.
            Lesson22Dbo.commit();

            // Read all the updated lines and display them on the console.
            List<Lesson22Dbo> dumpList = Lesson22Dbo.selectAllDynamic(firstUpdateLine);
            System.out.println();
            System.out.println("Displaying the updated rows");
            Lesson22Dbo.dump(dumpList);
        } catch (Exception e) {
            Lesson22Dbo.rollback();
            throw e;
        } finally {
            rdr.close();
        }
    }

}
